import ast
import asyncio
import csv
import datetime
import importlib.util as ilu
import inspect
import json
import math
import os
import psutil
import re
import struct
import sys
import time
import warnings
import webbrowser
import xml.etree.ElementTree as ET
from collections.abc import Callable
from datetime import timedelta
from pathlib import Path
from typing import Any, Dict, List, Optional, Set, Tuple, Union

import geopandas as gpd
import matplotlib as mpl
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import platform
import psycopg2
import pyodbc
import seaborn as sns
import subprocess
import sqlalchemy as sa
import threading
import tkinter as tk

from IPython.display import Markdown, display
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.gridspec import GridSpec
from PyPDF2 import PdfReader
from scipy.stats import kurtosis, norm, skew
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler
from sqlalchemy import URL, Column, MetaData, Table, create_engine, func, select, text
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import sessionmaker
from tkinter import filedialog, messagebox, ttk, Canvas
from tkinter.colorchooser import askcolor
from tkinter.scrolledtext import ScrolledText

from .pyswitch import Switch, switch


CONFIG_LANG = "en"
NORMAL_SIZE = (10, 6)
BIG_SIZE = (12, 8)
BG_COLOR = "#2d2d2d"
TEXT_COLOR = "#ffffff"
BTN_BG = "#3d3d3d"
HIGHLIGHT_COLOR = "#4e7cad"
CUSTOMGUI = False

def _custom_settings_(state: bool = False):
    global CUSTOMGUI
    try:
        CUSTOMGUI = state
    except Exception as e:
        print(f"Exception: {e}")

config = {"verbose": True, "default_timeout": 5, "counter": 0}


TRANSLATIONS = {}
_translations = {}

def _get_translation_path_(lang: str) -> Path:
    return Path(__file__).parent.parent / "lang" / f"{lang}.json"

def t(key: str, lang: str = None, return_raw: bool = False, **kwargs):
    
    if not key:
        return t("missing_translation_key").format(key=key)
    
    lang = lang or CONFIG_LANG
    
    entry = _translations.get(key, {})
    
    if isinstance(entry, str):
        translated = entry
    else:
        translated = entry.get(lang, f"[{key}]")
    
    if return_raw:
        return translated
    
    if kwargs:
        if isinstance(translated, dict) and "template" in translated:
            return translated["template"].format(**kwargs)
        elif isinstance(translated, str):
            try:
                return translated.format(**kwargs)
            except KeyError:
                return translated
    
    return translated

def load_user_translations(lang_path: str = "lang.json"):
    global _translations

    user_translations = {}
    path = Path(lang_path)

    if path.is_file():
        try:
            with open(path, "r", encoding="utf-8") as f:
                user_translations = json.load(f)
        except Exception as e:
            print(f"Warning: Error loading user translations: {str(e)}")

    _translations = TRANSLATIONS.copy()
    _translations.update(user_translations)

__LANGUAGES__ = [
    "af", "sq", "am", "ar", "hy", "as", "ay", "az", "bm", "eu", "be", "bn", "bho", 
    "bs", "bg", "ca", "ceb", "ny", "zh", "co", "hr", "cs", "da", "dv", 
    "doi", "nl", "en", "eo", "et", "ee", "tl", "fi", "fr", "fy", "gl", "ka", "de", 
    "el", "gn", "gu", "ht", "ha", "haw", "iw", "hi", "hmn", "hu", "is", "ig", "ilo", 
    "id", "ga", "it", "ja", "jw", "kn", "kk", "km", "rw", "gom", "ko", "kri", "ku", 
    "ckb", "ky", "lo", "la", "lv", "ln", "lt", "lg", "lb", "mk", "mai", "mg", "ms", 
    "ml", "mt", "mi", "mr", "lus", "mn", "my", "ne", "no", "or", "om", 
    "ps", "fa", "pl", "pt", "pa", "qu", "ro", "ru", "sm", "sa", "gd", "nso", "sr", 
    "st", "sn", "sd", "si", "sk", "sl", "so", "es", "su", "sw", "sv", "tg", "ta", 
    "tt", "te", "th", "ti", "ts", "tr", "tk", "ak", "uk", "ur", "ug", "uz", "vi", 
    "cy", "xh", "yi", "yo", "zu"
]

def _load_translations_(lang_to_load: str):
    """Carga las traducciones para un idioma específico"""
    global TRANSLATIONS, _translations
    
    translations_path = _get_translation_path_(lang_to_load)
    if translations_path.exists():
        try:
            with open(translations_path, encoding="utf-8") as f:
                TRANSLATIONS = json.load(f)
                _translations = TRANSLATIONS.copy()
                return True
        except Exception as e:
            print(f"Warning: Error loading translations for {lang_to_load}: {str(e)}")
            return False
    else:
        print(f"Warning: Translations not found for language: {lang_to_load}")
        return False

def set_language(lang: str):
    global CONFIG_LANG
    if TRANSLATIONS and lang not in __LANGUAGES__:
        raise ValueError(f"Language '{lang}' is not available.")
    CONFIG_LANG = lang
    _load_translations_(lang_to_load=lang)


TRANSLATIONS_PATH = _get_translation_path_(CONFIG_LANG)
if TRANSLATIONS_PATH.exists():
    with open(TRANSLATIONS_PATH, encoding="utf-8") as f:
        TRANSLATIONS = json.load(f)
        _translations = TRANSLATIONS.copy()
else:
    print("Warning: Default translations not found")


def _load_gui_config_():
    """Carga la configuración de la GUI desde un archivo JSON."""
    gui_json = "gui_config.json" if not CUSTOMGUI else "custom_gui_config.json"
    config_path = os.path.join(os.getcwd(), "gui_config.json")
    default_config = {
        "bg_color": "#2b2b2b",
        "text_color": "#ffffff",
        "btn_bg": "#3c3f41",
        "highlight_color": "#4e5254",
        "preview_bg": "#f0f0f0"
    }
    
    try:
        if os.path.exists(config_path):
            with open(config_path, 'r') as f:
                user_config = json.load(f)
                return {**default_config, **user_config}
    except:
        pass
    
    return default_config

def _save_gui_config_(config):
    """Guarda la configuración de la GUI en un archivo JSON."""
    config_path = os.path.join(os.getcwd(), "gui_config.json")
    try:
        with open(config_path, 'w') as f:
            json.dump(config, f, indent=4)
    except Exception as e:
        print(f"Error guardando configuración: {str(e)}")


def format_number(
    value: float, use_decimals: bool = True, decimals: int = 2, percent: bool = False
) -> str:
    if value is None:
        return "N/A"
    
    try:
        if isinstance(value, float) and np.isnan(value):
            return "N/A"
    except NameError:
        pass

    if percent:
        value *= 100

    if use_decimals:
        formatted = f"{value:,.{decimals}f}"
    else:
        formatted = f"{int(round(value)):,}"

    formatted = formatted.replace(",", "X").replace(".", ",").replace("X", ".")

    if percent:
        formatted += "%"

    return formatted


def is_jupyter_notebook():
    try:
        from IPython import get_ipython

        ip = get_ipython()
        if ip is None:
            return False

        shell_name = ip.__class__.__name__
        if shell_name == "ZMQInteractiveShell":
            return True
        else:
            return False
    except ImportError:
        return False

IN_JUPYTER = is_jupyter_notebook()


REGISTRY = {}

def register(name=None):
    """Decorator to register a function or class in the global REGISTRY."""
    def wrapper(fn):
        key = name or fn.__name__
        REGISTRY[key] = fn
        return fn
    return wrapper


def show_gui_popup(
    title, content, fig=None, plot_function=None, plot_args=None, 
    preview_mode=False, export_callback=None, table_data=None
):

    translations = {
        "copy": "copy",
        "close": "close", 
        "save": "save",
        "content": "content",
        "preview": "preview",
        "error_in_gui": "error_in_gui",
        "export": "export",
        "light_theme": "light_theme",
        "dark_theme": "dark_theme",
        "settings": "settings",
        "custom_inactive": "custom_inactive",
        "custom_active": "custom_active"
    }
    

    copy_text = Switch(translations["copy"])(
        lambda x: "copy", lambda: t("copy"),
        "default", lambda: "Copy"
    )
    
    close_text = Switch(translations["close"])(
        lambda x: "close", lambda: t("close"),
        "default", lambda: "Close"
    )
    
    save_text = Switch(translations["save"])(
        lambda x: "save", lambda: t("save"),
        "default", lambda: "Save"
    )
    
    content_text = Switch(translations["content"])(
        lambda x: "content", lambda: t("content"),
        "default", lambda: "Content"
    )
    
    preview_text = Switch(translations["preview"])(
        lambda x: "preview", lambda: t("preview"),
        "default", lambda: "Preview"
    )
    
    gui_error_text = Switch(translations["error_in_gui"])(
        lambda x: "error_in_gui", lambda: t("error_in_gui"),
        "default", lambda: "GUI Error"
    )
    
    export_text = Switch(translations["export"])(
        lambda x: "export", lambda: t("export"),
        "default", lambda: "Export"
    )
    
    light_theme_text = Switch(translations["light_theme"])(
        lambda x: "light_theme", lambda: t("light_theme"),
        "default", lambda: "Light Theme"
    )
    
    dark_theme_text = Switch(translations["dark_theme"])(
        lambda x: "dark_theme", lambda: t("dark_theme"),
        "default", lambda: "Dark Theme"
    )

    custom_inactive_text = Switch(translations["custom_inactive"])(
        lambda x: "custom_inactive", lambda: t("custom_inactive"),
        "default", lambda: "Custom Color Disable"
    )

    custom_active_text = Switch(translations["custom_active"])(
        lambda x: "custom_active", lambda: t("custom_active"),
        "default", lambda: "Custom Color Enable"
    )
    
    settings_text = Switch(translations["settings"])(
        lambda x: "settings", lambda: t("settings"),
        "default", lambda: "Settings"
    )


    config = _load_gui_config_()
    BG_COLOR = config.get("bg_color", "#2b2b2b")
    TEXT_COLOR = config.get("text_color", "#ffffff")
    BTN_BG = config.get("btn_bg", "#3c3f41")
    HIGHLIGHT_COLOR = config.get("highlight_color", "#4e5254")
    PREVIEW_BG = config.get("preview_bg", "#f0f0f0")


    if IN_JUPYTER:
        mpl.use("module://ipykernel.pylab.backend_inline")
    else:
        mpl.use("Agg")

    current_fig = fig
    if plot_function is not None:
        if plot_args is None:
            plot_args = {}
        current_fig = plot_function(**plot_args)

    if preview_mode:
        if current_fig is not None:
            current_fig.patch.set_facecolor(PREVIEW_BG)
            for ax in current_fig.get_axes():
                ax.set_facecolor(PREVIEW_BG)
                ax.title.set_color(TEXT_COLOR)
                ax.xaxis.label.set_color(TEXT_COLOR)
                ax.yaxis.label.set_color(TEXT_COLOR)
                ax.tick_params(colors=TEXT_COLOR)
                for spine in ax.spines.values():
                    spine.set_color(TEXT_COLOR)
        return current_fig


    window = tk.Tk()
    window.title(title)
    window.state("zoomed")
    window.configure(bg=BG_COLOR)


    style = ttk.Style()
    style.theme_use("clam")
    style.configure("Dark.TFrame", background=BG_COLOR)
    style.configure(
        "Dark.TLabel", background=BG_COLOR, foreground=TEXT_COLOR, font=("Consolas", 10)
    )
    style.configure(
        "Dark.TButton", background=BTN_BG, foreground=TEXT_COLOR, borderwidth=1
    )
    style.map("Dark.TButton", background=[("active", HIGHLIGHT_COLOR)])


    window.grid_rowconfigure(0, weight=1)
    window.grid_columnconfigure(0, weight=1)

    main_frame = ttk.Frame(window, style="Dark.TFrame")
    main_frame.grid(row=0, column=0, sticky="nsew", padx=10, pady=10)
    main_frame.grid_rowconfigure(0, weight=1)
    main_frame.grid_columnconfigure(0, weight=1)


    notebook = ttk.Notebook(main_frame)
    notebook.grid(row=0, column=0, sticky="nsew", pady=(0, 10))


    doc_frame = ttk.Frame(notebook, style="Dark.TFrame")
    notebook.add(doc_frame, text=content_text)

    text_area = ScrolledText(
        doc_frame,
        wrap=tk.WORD,
        font=("Consolas", 10),
        bg=BG_COLOR,
        fg=TEXT_COLOR,
        insertbackground=TEXT_COLOR,
        selectbackground=HIGHLIGHT_COLOR,
    )
    text_area.pack(expand=True, fill="both", padx=5, pady=5)
    text_area.insert(tk.END, content)
    text_area.config(state="disabled")


    current_fig = fig
    canvas = None

    if fig is not None or plot_function is not None:

        graph_frame = ttk.Frame(notebook, style="Dark.TFrame")
        notebook.add(graph_frame, text=preview_text)

        if plot_function is not None:
            if plot_args is None:
                plot_args = {}
            current_fig = plot_function(**plot_args)

        if current_fig is not None:
            current_fig.patch.set_facecolor(PREVIEW_BG)
            for ax in current_fig.get_axes():
                ax.set_facecolor(PREVIEW_BG)
                ax.title.set_color("black")
                ax.xaxis.label.set_color("black")
                ax.yaxis.label.set_color("black")
                ax.tick_params(colors="black")
                for spine in ax.spines.values():
                    spine.set_color("black")

            canvas = FigureCanvasTkAgg(current_fig, master=graph_frame)
            canvas.draw()
            canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True, padx=5, pady=5)


    def copy_to_clipboard():
        window.clipboard_clear()
        window.clipboard_append(content)
        window.update()

    def save_image():
        if current_fig is not None:
            filepath = filedialog.asksaveasfilename(
                defaultextension=".png",
                filetypes=[
                    ("PNG files", "*.png"),
                    ("JPEG files", "*.jpg"),
                    ("All files", "*.*"),
                ],
            )
            if filepath:
                current_fig.savefig(filepath, bbox_inches="tight", dpi=300)

    def export_data():
        if export_callback:
            format_menu = tk.Menu(window, tearoff=0)
            formats = ["csv", "json", "excel", "xml", "sql", "parquet", "html"]
            
            for fmt in formats:
                format_menu.add_command(
                    label=fmt.upper(), 
                    command=lambda f=fmt: export_callback(f)
                )
            
            export_btn = btn_frame.winfo_children()[2]
            x = export_btn.winfo_rootx()
            y = export_btn.winfo_rooty() + export_btn.winfo_height()
            format_menu.post(x, y)

    def toggle_theme():
        nonlocal BG_COLOR, TEXT_COLOR, BTN_BG, HIGHLIGHT_COLOR, PREVIEW_BG
        
        theme_config = switch(
            BG_COLOR, {
                "#ffffff": {
                    "bg_color": "#2b2b2b",
                    "text_color": "#ffffff", 
                    "btn_bg": "#3c3f41",
                    "highlight_color": "#4e5254",
                    "preview_bg": "#f0f0f0"
                },
                "#2b2b2b": {
                    "bg_color": "#ffffff",
                    "text_color": "#000000",
                    "btn_bg": "#f0f0f0", 
                    "highlight_color": "#e0e0e0",
                    "preview_bg": "#f0f0f0"
                },
                "default": {
                "bg_color": "#2b2b2b",
                "text_color": "#ffffff",
                "btn_bg": "#3c3f41",
                "highlight_color": "#4e5254",
                "preview_bg": "#f0f0f0"
            }
            }
        )
        
        _save_gui_config_(theme_config)
        
        on_close()
        show_gui_popup(
            title, content, fig, plot_function, plot_args, 
            False, export_callback, table_data
        )

    def custom_mng():
        state = not CUSTOMGUI
        _custom_settings_(state)

    def open_settings():
        settings_window = tk.Toplevel(window)
        settings_window.title("Configuración de GUI")
        settings_window.geometry("400x300")
        settings_window.configure(bg=BG_COLOR)
        
        ttk.Label(settings_window, text="Configuración de colores", style="Dark.TLabel").pack(pady=10)

        user_bg_color = askcolor(color="FFFFFF", title="Choose background color")
        user_text_color = askcolor(color="FFFFFF", title="Choose text color")
        user_btn_bg = askcolor(color="FFFFFF", title="Choose button background color")
        user_highlight_color = askcolor(color="FFFFFF", title="Choose highlight color")
        user_preview_bg = askcolor(color="FFFFFF", title="Choose preview background color")

        custom_config = {
                    "bg_color": user_bg_color[1],
                    "text_color": user_text_color[1], 
                    "btn_bg": user_btn_bg[1],
                    "highlight_color": user_highlight_color[1],
                    "preview_bg": user_preview_bg[1]
                }
        
        def save_custom_colors():
            """Guarda la configuración de la GUI en un archivo JSON."""
            config_path = os.path.join(os.getcwd(), "custom_gui_config.json")
            try:
                with open(config_path, 'w') as f:
                    json.dump(custom_config, f, indent=4)
                    _custom_settings_(True)
            except Exception as e:
                print(f"Error guardando configuración: {str(e)}")
        
        ttk.Button(settings_window, text="Guardar", command=save_custom_colors, style="Dark.TButton").pack(pady=10)

    def on_close():
        if current_fig is not None:
            plt.close(current_fig)
        window.quit()
        window.destroy()


    btn_frame = ttk.Frame(main_frame, style="Dark.TFrame")
    btn_frame.grid(row=1, column=0, sticky="ew", pady=(0, 5))

    btn_frame.grid_columnconfigure(0, weight=1)
    btn_frame.grid_columnconfigure(1, weight=1)
    btn_frame.grid_columnconfigure(2, weight=1)
    btn_frame.grid_columnconfigure(3, weight=1)
    btn_frame.grid_columnconfigure(4, weight=1)


    action_btn = ttk.Button(
        btn_frame, text=copy_text, command=copy_to_clipboard, style="Dark.TButton"
    )
    action_btn.grid(row=0, column=0, padx=5, sticky="w")


    if export_callback:
        export_btn = ttk.Button(
            btn_frame, text=export_text, command=export_data, style="Dark.TButton"
        )
        export_btn.grid(row=0, column=1, padx=5)


    custom_event_btn_text = switch(CUSTOMGUI,{
        False: custom_inactive_text,
        True: custom_active_text,
        "default": custom_inactive_text}
    )
    
    custom_event_btn = ttk.Button(
        btn_frame, text=custom_event_btn_text, 
        command=custom_mng, style="Dark.TButton"
    )
    if CUSTOMGUI:
        custom_event_btn.grid(row=0, column=2, padx=5)
    else:
        custom_event_btn.grid(row=0, column=2, padx=2)
    

    theme_btn_text = switch(BG_COLOR,{
        "#ffffff": dark_theme_text,
        "#2b2b2b": light_theme_text,
        "default": light_theme_text}
    )
    
    theme_btn = ttk.Button(
        btn_frame, text=theme_btn_text, 
        command=toggle_theme, style="Dark.TButton"
    )
    theme_btn.grid(row=0, column=2, padx=2)

    if CUSTOMGUI:
        theme_btn.grid_forget()
    


    settings_btn = ttk.Button(
        btn_frame, text=settings_text, command=open_settings, style="Dark.TButton"
    )
    settings_btn.grid(row=0, column=3, padx=5)


    close_btn = ttk.Button(
        btn_frame, text=close_text, command=on_close, style="Dark.TButton"
    )
    close_btn.grid(row=0, column=4, padx=5, sticky="e")


    def on_tab_change(event):
        tab_action = Switch(notebook.index("current"))(
            lambda x: 1, lambda: {"action_text": save_text, "action_command": save_image},
            "default", lambda: {"action_text": copy_text, "action_command": copy_to_clipboard}
        )
        action_btn.config(text=tab_action["action_text"], command=tab_action["action_command"])

    notebook.bind("<<NotebookTabChanged>>", on_tab_change)

    def _jup_func_():
        import ipywidgets as widgets

        output = widgets.Output(),
        display(output),

        def run_in_jupyter():
            with output:
                try:
                    window.mainloop()
                except Exception as e:
                    print(f"{gui_error_text}: {str(e)}")

        return window.after(100, run_in_jupyter)


    if fig is not None or plot_function is not None:
        if notebook.index("current") == 1:
            action_btn.config(text=save_text, command=save_image)


    if IN_JUPYTER:
         _jup_func_()
    else:
        window.mainloop()

    if current_fig is not None:
        plt.close(current_fig)

def show_alert_popup(type: str, message: str, detail: str = None):
    """Muestra un popup de advertencia o error con colores predefinidos."""


    type = type.strip().lower()
    if type not in ("warning", "error"):
        raise ValueError("type debe ser 'warning' o 'error'")


    if type == "warning":
        bg_color = "#ffd9a3cc"   # naranja muy claro
        fg_color = "#663c00"   # marrón/naranja oscuro
        accent_color = "#ff9800"
        title = "Warning"
    else:
        bg_color = "#ee978d"   # rojo claro
        fg_color = "#611a15"   # rojo oscuro
        accent_color = "#f44336"
        title = "Error"


    win = tk.Toplevel()
    win.title(title)
    win.configure(bg=bg_color)
    win.resizable(False, False)

    style = ttk.Style()
    style.theme_use("clam")
    style.configure("Custom.TButton", background=accent_color, foreground="white")
    style.map("Custom.TButton", background=[("active", fg_color)])


    lbl_msg = ttk.Label(win, text=message, background=bg_color, foreground=fg_color, font=("Segoe UI", 11, "bold"))
    lbl_msg.pack(padx=15, pady=(15,5), anchor="w")


    btn_frame = ttk.Frame(win)
    btn_frame.pack(fill="x", padx=10, pady=(10, 10))


    def copy_message():
        win.clipboard_clear()
        win.clipboard_append(message)
        win.update()

    btn_copy = ttk.Button(btn_frame, text="Copiar mensaje", command=copy_message, style="Custom.TButton")
    btn_copy.pack(side="left", padx=5)


    def close_win():
        win.destroy()

    btn_close = ttk.Button(btn_frame, text="Cerrar", command=close_win, style="Custom.TButton")
    btn_close.pack(side="right", padx=5)


    if detail:
        detail_frame = ttk.Frame(win)
        detail_frame.pack(fill="both", expand=True, padx=10, pady=(5,10))


        def toggle_detail():
            if not hasattr(toggle_detail, "shown"):
                toggle_detail.shown = False

            if not toggle_detail.shown:
                text_detail.pack(fill="both", expand=True, pady=(5,5))
                btn_copy_detail.pack(side="left", padx=5)
                btn_detail.config(text="Ocultar detalle")
            else:
                text_detail.pack_forget()
                btn_copy_detail.pack_forget()
                btn_detail.config(text="Mostrar detalle")

            toggle_detail.shown = not toggle_detail.shown

        btn_detail = ttk.Button(detail_frame, text="Mostrar detalle", command=toggle_detail, style="Custom.TButton")
        btn_detail.pack(fill="x", pady=(0,5))


        text_detail = ScrolledText(detail_frame, wrap="word", height=8, background="white", foreground="black")
        text_detail.insert("1.0", detail)
        text_detail.configure(state="disabled")


        def copy_detail():
            win.clipboard_clear()
            win.clipboard_append(detail)
            win.update()

        btn_copy_detail = ttk.Button(detail_frame, text="Copiar detalle", command=copy_detail, style="Custom.TButton")


    win.update_idletasks()
    w = win.winfo_width()
    h = win.winfo_height()
    x = (win.winfo_screenwidth() // 2) - (w // 2)
    y = (win.winfo_screenheight() // 2) - (h // 2)
    win.geometry(f"+{x}+{y}")

    win.transient()   # Se mantiene encima
    win.grab_set()    # Bloquea interacción con otras ventanas
    win.mainloop()

def show_yesno_popup(title, message):
        """Muestra un popup con botones Sí/No"""
        root = tk.Tk()
        root.withdraw()  # Ocultar ventana principal
        root.attributes('-topmost', True)  # Traer al frente
        result = messagebox.askyesno(t(title), t(message))
        root.destroy()
        return result



import ast, sys, os
from pathlib import Path
from typing import Union, Dict, Set, List, Tuple
from .shared import t, show_gui_popup, IN_JUPYTER, display, Markdown
from .pyswitch import switch


class PythonFileChecker:
    def __init__(self):
        self.errors: List[Dict[str, str]] = []
        self.imported_names: Set[str] = set()
        self.defined_names: Set[str] = set()
        self.import_lines: Dict[str, int] = {}
        self.syntax_errors: List[Dict[str, str]] = []
        self.exception_handlers: Set[str] = set()
        self.python_exceptions = [
            "BaseException", "SystemExit", "KeyboardInterrupt", "GeneratorExit", "Exception",
            "ArithmeticError", "FloatingPointError", "OverflowError", "ZeroDivisionError",
            "AssertionError", "AttributeError", "BufferError", "EOFError", "ImportError",
            "ModuleNotFoundError", "LookupError", "IndexError", "KeyError", "MemoryError",
            "NameError", "UnboundLocalError", "OSError", "BlockingIOError", "ChildProcessError",
            "ConnectionError", "BrokenPipeError", "ConnectionAbortedError", "ConnectionRefusedError",
            "ConnectionResetError", "FileExistsError", "FileNotFoundError", "InterruptedError",
            "IsADirectoryError", "NotADirectoryError", "PermissionError", "ProcessLookupError",
            "TimeoutError", "ReferenceError", "RuntimeError", "NotImplementedError", "RecursionError",
            "StopAsyncIteration", "StopIteration", "SyntaxError", "IndentationError", "TabError",
            "SystemError", "TypeError", "ValueError", "UnicodeError", "UnicodeDecodeError",
            "UnicodeEncodeError", "UnicodeTranslateError", "Warning", "DeprecationWarning",
            "PendingDeprecationWarning", "RuntimeWarning", "SyntaxWarning", "UserWarning",
            "FutureWarning", "ImportWarning", "UnicodeWarning", "BytesWarning", "ResourceWarning"
        ]

    def check_file(self, file_path: Union[str, Path]) -> bool:
        """Check a Python file for all possible errors without stopping"""
        self._reset_state()
        
        if not os.path.exists(file_path):
            self._add_error(0, t("file_not_found").format(file_path=file_path), "FileNotFound")
            return False

        try:
            with open(file_path, "r", encoding="utf-8") as f:
                source_code = f.read()
                source_lines = source_code.splitlines()
        except UnicodeDecodeError:
            self._add_error(0, t("encoding_error"), "EncodingError")
            return False
        except Exception as e:
            self._add_error(0, t("file_read_error").format(error=str(e)), "IOError")
            return False


        tree = self._parse_code(source_code, source_lines)
        
        if tree is not None:
            self._analyze_tree(tree, source_lines)
        else:
            self._incremental_analysis(source_code, source_lines)


        self.errors.extend(self.syntax_errors)
        
        return len(self.errors) == 0

    def _reset_state(self) -> None:
        """Reset all state variables"""
        self.errors.clear()
        self.imported_names.clear()
        self.defined_names.clear()
        self.import_lines.clear()
        self.syntax_errors.clear()
        self.exception_handlers.clear()

    def _parse_code(self, source_code: str, source_lines: List[str]) -> ast.AST:
        """Parse the source code and handle syntax errors"""
        try:
            return ast.parse(source_code)
        except SyntaxError as e:
            self._add_syntax_error(e, source_lines)
            try:
                return ast.parse(source_code, mode="exec")
            except SyntaxError as e2:
                self._add_syntax_error(e2, source_lines)
            except Exception:
                pass
        return None

    def _analyze_tree(self, tree: ast.AST, source_lines: List[str]) -> None:
        """Analyze the AST tree for various issues"""
        try:

            for node in ast.walk(tree):
                for child in ast.iter_child_nodes(node):
                    setattr(child, "parent", node)

            self._collect_imports_and_definitions(tree)
            self._check_exception_handling(tree, source_lines)
            self._check_undefined_names(tree, source_lines)
            self._check_other_issues(tree, source_lines)

        except Exception as e:
            self._add_error(0, t("analysis_error").format(msg=str(e)), "AnalysisError")

    def _add_syntax_error(self, error: SyntaxError, source_lines: List[str]) -> None:
        """Helper to add syntax errors with proper formatting"""
        line_content = ""
        if error.lineno and error.lineno <= len(source_lines):
            line_content = source_lines[error.lineno - 1].strip()
            
        self.syntax_errors.append({
            "line": error.lineno or 0,
            "message": t("syntax_error").format(msg=error.msg),
            "type": "SyntaxError",
            "context": line_content,
        })

    def _incremental_analysis(self, source_code: str, source_lines: List[str]) -> None:
        """Try to analyze the file incrementally when full parsing fails"""

        for i, line in enumerate(source_lines, 1):
            line = line.strip()
            if not line or line.startswith(('#', '"""', "'''")):
                continue
                
            try:
                ast.parse(line)
            except SyntaxError as e:
                self._add_syntax_error(e, source_lines)
            except Exception:
                pass


        self._analyze_code_blocks(source_code, source_lines)

    def _analyze_code_blocks(self, source_code: str, source_lines: List[str]) -> None:
        """Try to analyze blocks of code when full parsing fails"""
        blocks = self._split_into_blocks(source_lines)
        
        for block_content, start_line in blocks:
            try:
                tree = ast.parse(block_content)

                for node in ast.walk(tree):
                    for child in ast.iter_child_nodes(node):
                        setattr(child, "parent", node)

                self._collect_imports_and_definitions(tree)
                self._check_exception_handling_with_offset(tree, source_lines, start_line - 1)
                self._check_undefined_names_with_offset(tree, source_lines, start_line - 1)
                self._check_other_issues_with_offset(tree, source_lines, start_line - 1)

            except SyntaxError as e:
                e.lineno = (e.lineno or 0) + start_line - 1
                self._add_syntax_error(e, source_lines)
            except Exception:
                pass

    def _split_into_blocks(self, source_lines: List[str]) -> List[Tuple[str, int]]:
        """Split source code into logical blocks"""
        blocks = []
        current_block = []
        
        for i, line in enumerate(source_lines):
            if line.strip() == "" and current_block:
                blocks.append(("\n".join(current_block), i - len(current_block) + 1))
                current_block = []
            elif line.strip() != "":
                current_block.append(line)
                
        if current_block:
            blocks.append(("\n".join(current_block), len(source_lines) - len(current_block) + 1))
            
        return blocks

    def _check_exception_handling(self, tree: ast.AST, source_lines: List[str]) -> None:
        """Check for proper exception handling"""
        for node in ast.walk(tree):
            node_type = type(node).__name__
            switch(node_type, {
                "Try": self._analyze_try_block(node, source_lines),
                "Raise": self._analyze_raise_statement(node, source_lines),
                "default": None
            })

    def _check_exception_handling_with_offset(self, tree: ast.AST, source_lines: List[str], line_offset: int) -> None:
        """Check exception handling with line number adjustment"""
        for node in ast.walk(tree):
            node_type = type(node).__name__
            switch(node_type, {
                "Try": self._analyze_try_block_with_offset(node, source_lines, line_offset),
                "Raise": self._analyze_raise_statement_with_offset(node, source_lines, line_offset),
                "default": None
            })

    def _analyze_try_block(self, node: ast.Try, source_lines: List[str]) -> None:
        """Analyze a try block for exception handling issues"""
        for handler in node.handlers:
            if handler.type:

                handler_type = type(handler.type).__name__
                switch(handler_type, {
                    "Name": self.exception_handlers.add(handler.type.id),
                    "Tuple": [self.exception_handlers.add(elt.id) for elt in handler.type.elts 
                                     if isinstance(elt, ast.Name)],
                    "default": None
                })
            

            if len(handler.body) == 1 and isinstance(handler.body[0], ast.Pass):
                line_content = source_lines[handler.lineno - 1].strip() if handler.lineno <= len(source_lines) else ""
                self._add_error(
                    handler.lineno,
                    t("empty_except_handler"),
                    "ExceptionWarning",
                    line_content,
                )

    def _analyze_try_block_with_offset(self, node: ast.Try, source_lines: List[str], line_offset: int) -> None:
        """Analyze try block with line offset"""
        for handler in node.handlers:
            if handler.type:
                handler_type = type(handler.type).__name__
                switch(handler_type, {
                    "Name": self.exception_handlers.add(handler.type.id),
                    "Tuple": [self.exception_handlers.add(elt.id) for elt in handler.type.elts 
                                     if isinstance(elt, ast.Name)],
                    "default": None
                })
            
            if len(handler.body) == 1 and isinstance(handler.body[0], ast.Pass):
                adjusted_line = handler.lineno + line_offset
                line_content = source_lines[adjusted_line - 1].strip() if adjusted_line <= len(source_lines) else ""
                self._add_error(
                    adjusted_line,
                    t("empty_except_handler"),
                    "ExceptionWarning",
                    line_content,
                )

    def _analyze_raise_statement(self, node: ast.Raise, source_lines: List[str]) -> None:
        """Analyze raise statements for potential issues"""
        if node.exc is None:

            line_content = source_lines[node.lineno - 1].strip() if node.lineno <= len(source_lines) else ""
            self._add_error(
                node.lineno,
                t("bare_raise_outside_handler"),
                "ExceptionWarning",
                line_content,
            )
        elif isinstance(node.exc, ast.Call):

            if isinstance(node.exc.func, ast.Name):
                exception_name = node.exc.func.id
                if exception_name not in self.python_exceptions:
                    line_content = source_lines[node.lineno - 1].strip() if node.lineno <= len(source_lines) else ""
                    self._add_error(
                        node.lineno,
                        t("unknown_exception_type").format(name=exception_name),
                        "ExceptionWarning",
                        line_content,
                    )

    def _analyze_raise_statement_with_offset(self, node: ast.Raise, source_lines: List[str], line_offset: int) -> None:
        """Analyze raise statement with line offset"""
        if node.exc is None:
            adjusted_line = node.lineno + line_offset
            line_content = source_lines[adjusted_line - 1].strip() if adjusted_line <= len(source_lines) else ""
            self._add_error(
                adjusted_line,
                t("bare_raise_outside_handler"),
                "ExceptionWarning",
                line_content,
            )
        elif isinstance(node.exc, ast.Call):
            if isinstance(node.exc.func, ast.Name):
                exception_name = node.exc.func.id
                if exception_name not in self.python_exceptions:
                    adjusted_line = node.lineno + line_offset
                    line_content = source_lines[adjusted_line - 1].strip() if adjusted_line <= len(source_lines) else ""
                    self._add_error(
                        adjusted_line,
                        t("unknown_exception_type").format(name=exception_name),
                        "ExceptionWarning",
                        line_content,
                    )

    def _check_undefined_names_with_offset(self, tree: ast.AST, source_lines: List[str], line_offset: int) -> None:
        """Check for undefined names with line number adjustment"""
        defined = self.defined_names.union(self.imported_names)
        builtins = set(dir(__builtins__)).union({
            'self', 'cls', '_', 'print', 'open', 
            'ImportError', 'globals', 'Exception'
        })

        for node in ast.walk(tree):
            try:
                if isinstance(node, ast.Name) and isinstance(node.ctx, ast.Load):
                    name = node.id
                    if (name not in defined and 
                        name not in builtins and 
                        not name.startswith('_')):
                        
                        is_attribute = False
                        if hasattr(node, "parent"):
                            parent = node.parent
                            if (isinstance(parent, ast.Attribute) and 
                                parent.value == node):
                                is_attribute = True

                        if not is_attribute:
                            adjusted_line = node.lineno + line_offset
                            line_content = (
                                source_lines[adjusted_line - 1].strip()
                                if adjusted_line <= len(source_lines)
                                else ""
                            )
                            self._add_error(
                                adjusted_line,
                                t("undefined_name").format(name=name),
                                "NameError",
                                line_content,
                            )
            except Exception:
                continue

    def _check_other_issues_with_offset(self, tree: ast.AST, source_lines: List[str], line_offset: int) -> None:
        """Check for other potential issues with line number adjustment"""
        imported_names = set(self.import_lines.keys())
        all_used_names = set()
        assigned_but_unused = {}

        bad_comparisons = {
            "Eq": {  # ==
                "True": t("redundant_true"),
                "False": t("redundant_false"),
                "None": t("none_comparison"),
            },
            "NotEq": {  # !=
                "True": t("not_true_recommendation"),
                "False": t("not_false_recommendation"),
            },
        }

        for node in ast.walk(tree):
            try:
                node_type = type(node).__name__
                switch(node_type, {
                    "Name": self._handle_name_node(node, imported_names, all_used_names) if isinstance(node.ctx, ast.Load) else None,
                    "Assign": self._handle_assign_node(node, assigned_but_unused, line_offset),
                    "Compare": self._handle_compare_node(node, bad_comparisons, source_lines, line_offset),
                    "FunctionDef": self._handle_function_def_node(node, source_lines, line_offset),
                    "ClassDef": self._handle_class_def_node(node, source_lines, line_offset),
                    "AsyncFunctionDef": self._handle_function_def_node(node, source_lines, line_offset),
                    "default": None
                })
            except Exception:
                continue


        for name in imported_names:
            line = self.import_lines.get(name, 0)
            line_content = source_lines[line - 1].strip() if line > 0 and line <= len(source_lines) else ""
            self._add_error(
                line,
                f"Unused import '{name}'",
                "StyleWarning",
                line_content,
            )


        for name, line in assigned_but_unused.items():
            if name not in all_used_names:
                line_content = source_lines[line - 1].strip() if line <= len(source_lines) else ""
                self._add_error(
                    line,
                    f"Unused variable '{name}'",
                    "StyleWarning",
                    line_content,
                )

    def _handle_name_node(self, node: ast.Name, imported_names: Set[str], all_used_names: Set[str]) -> None:
        """Handle Name node in switch statement"""
        all_used_names.add(node.id)
        if node.id in imported_names:
            imported_names.remove(node.id)

    def _handle_assign_node(self, node: ast.Assign, assigned_but_unused: Dict[str, int], line_offset: int) -> None:
        """Handle Assign node in switch statement"""
        for target in node.targets:
            target_type = type(target).__name__
            switch(target_type, {
                "Name": assigned_but_unused.__setitem__(target.id, node.lineno + line_offset),
                "Tuple": [assigned_but_unused.__setitem__(elt.id, node.lineno + line_offset) 
                                for elt in target.elts if isinstance(elt, ast.Name)],
                "default": None
            })

    def _handle_compare_node(self, node: ast.Compare, bad_comparisons: Dict, source_lines: List[str], line_offset: int) -> None:
        """Handle Compare node in switch statement"""
        if node.ops:
            op_type = type(node.ops[0]).__name__
            if op_type in bad_comparisons:
                for comparator in node.comparators:
                    if isinstance(comparator, ast.Constant):
                        const_value = str(comparator.value)
                        if const_value in bad_comparisons[op_type]:
                            adjusted_line = node.lineno + line_offset
                            line_content = source_lines[adjusted_line - 1].strip() if adjusted_line <= len(source_lines) else ""
                            self._add_error(
                                adjusted_line,
                                bad_comparisons[op_type][const_value],
                                "StyleWarning",
                                line_content,
                            )

    def _handle_function_def_node(self, node: Union[ast.FunctionDef, ast.AsyncFunctionDef], 
                                 source_lines: List[str], line_offset: int) -> None:
        """Handle FunctionDef and AsyncFunctionDef nodes in switch statement"""
        adjusted_line = node.lineno + line_offset
        line_content = source_lines[adjusted_line - 1].strip() if adjusted_line <= len(source_lines) else ""
        
        if not node.body:
            self._add_error(
                adjusted_line,
                t("empty_block").format(type=t("function")),
                "IndentationWarning",
                line_content,
            )
        elif len(node.body) == 1 and isinstance(node.body[0], ast.Pass):
            self._add_error(
                adjusted_line,
                t("empty_block_with_pass").format(type=t("function")),
                "IndentationWarning",
                line_content,
            )

    def _handle_class_def_node(self, node: ast.ClassDef, source_lines: List[str], line_offset: int) -> None:
        """Handle ClassDef node in switch statement"""
        adjusted_line = node.lineno + line_offset
        line_content = source_lines[adjusted_line - 1].strip() if adjusted_line <= len(source_lines) else ""
        
        if not node.body:
            self._add_error(
                adjusted_line,
                t("empty_block").format(type=t("class")),
                "IndentationWarning",
                line_content,
            )
        elif len(node.body) == 1 and isinstance(node.body[0], ast.Pass):
            self._add_error(
                adjusted_line,
                t("empty_block_with_pass").format(type=t("class")),
                "IndentationWarning",
                line_content,
            )

    def _add_error(self, line: int, message: str, error_type: str, context: str = None):
        """Helper to add errors with translation support"""
        self.errors.append({
            "line": line,
            "message": message,
            "type": error_type,
            "context": context,
        })

    def _collect_imports_and_definitions(self, tree: ast.AST) -> None:
        """Collect all imported and defined names in the AST."""
        for node in ast.walk(tree):
            node_type = type(node).__name__
            switch(node_type, {
                "Import": self._handle_import_node(node),
                "ImportFrom": self._handle_import_from_node(node),
                "FunctionDef": self.defined_names.add(node.name),
                "AsyncFunctionDef": self.defined_names.add(node.name),
                "ClassDef": self.defined_names.add(node.name),
                "Assign": self._handle_assign_definition_node(node),
                "default": None
            })

    def _handle_import_node(self, node: ast.Import) -> None:
        """Handle Import node in switch statement"""
        for alias in node.names:
            self.imported_names.add(alias.name)
            self.import_lines[alias.name] = node.lineno
            if alias.asname:
                self.imported_names.add(alias.asname)
                self.import_lines[alias.asname] = node.lineno

    def _handle_import_from_node(self, node: ast.ImportFrom) -> None:
        """Handle ImportFrom node in switch statement"""
        module = node.module or ""
        for alias in node.names:
            full_name = f"{module}.{alias.name}" if module else alias.name
            self.imported_names.add(full_name)
            self.import_lines[full_name] = node.lineno
            if alias.asname:
                self.imported_names.add(alias.asname)
                self.import_lines[alias.asname] = node.lineno
            else:
                self.imported_names.add(alias.name)
                self.import_lines[alias.name] = node.lineno

    def _handle_assign_definition_node(self, node: ast.Assign) -> None:
        """Handle Assign node for definitions in switch statement"""
        for target in node.targets:
            target_type = type(target).__name__
            switch(target_type, {
                "Name": self.defined_names.add(target.id),
                "Tuple": [self.defined_names.add(elt.id) for elt in target.elts 
                                 if isinstance(elt, ast.Name)],
                "default": None
            })

    def _check_undefined_names(self, tree: ast.AST, source_lines: List[str]) -> None:
        """Check for undefined names in the AST."""
        defined = self.defined_names.union(self.imported_names)
        builtins = set(dir(__builtins__)).union({
            'self', 'cls', '_', 'print', 'open', 
            'ImportError', 'globals', 'Exception'
        })

        for node in ast.walk(tree):
            try:
                if isinstance(node, ast.Name) and isinstance(node.ctx, ast.Load):
                    name = node.id
                    if (name not in defined and 
                        name not in builtins and 
                        not name.startswith('_')):
                        
                        is_attribute = False
                        if hasattr(node, "parent"):
                            parent = node.parent
                            if (isinstance(parent, ast.Attribute) and 
                                parent.value == node):
                                is_attribute = True

                        if not is_attribute:
                            line_content = (
                                source_lines[node.lineno - 1].strip()
                                if node.lineno <= len(source_lines)
                                else ""
                            )
                            self._add_error(
                                node.lineno,
                                t("undefined_name").format(name=name),
                                "NameError",
                                line_content,
                            )
            except Exception:
                continue

    def _check_other_issues(self, tree: ast.AST, source_lines: List[str]) -> None:
        """Check for other potential issues in the code."""
        imported_names = set(self.import_lines.keys())
        all_used_names = set()
        assigned_but_unused = {}

        bad_comparisons = {
            "Eq": {  # ==
                "True": t("redundant_true"),
                "False": t("redundant_false"),
                "None": t("none_comparison"),
            },
            "NotEq": {  # !=
                "True": t("not_true_recommendation"),
                "False": t("not_false_recommendation"),
            },
        }

        for node in ast.walk(tree):
            try:
                node_type = type(node).__name__
                switch(node_type, {
                    "Name": self._handle_name_node(node, imported_names, all_used_names) if isinstance(node.ctx, ast.Load) else None,
                    "Assign": self._handle_assign_node(node, assigned_but_unused, 0),
                    "Compare": self._handle_compare_node(node, bad_comparisons, source_lines, 0),
                    "FunctionDef": self._handle_function_def_node(node, source_lines, 0),
                    "ClassDef": self._handle_class_def_node(node, source_lines, 0),
                    "AsyncFunctionDef": self._handle_function_def_node(node, source_lines, 0),
                    "default": None
                })
            except Exception:
                continue


        for name in imported_names:
            line = self.import_lines.get(name, 0)
            line_content = source_lines[line - 1].strip() if line > 0 and line <= len(source_lines) else ""
            self._add_error(
                line,
                f"Unused import '{name}'",
                "StyleWarning",
                line_content,
            )


        for name, line in assigned_but_unused.items():
            if name not in all_used_names:
                line_content = source_lines[line - 1].strip() if line <= len(source_lines) else ""
                self._add_error(
                    line,
                    f"Unused variable '{name}'",
                    "StyleWarning",
                    line_content,
                )

    def get_errors(self) -> List[Dict[str, str]]:
        """Return the list of errors found."""
        return sorted(self.errors, key=lambda e: e["line"])

def check_syntax(file_path: Union[str, Path]) -> None:
    """Check a Python file and show all errors found."""
    checker = PythonFileChecker()
    checker.check_file(file_path)

    errors = checker.get_errors()

    if not errors:
        if IN_JUPYTER:
            display(Markdown(f"**{t('syntax_check_title')}**\n\n{t('no_errors_found')}"))
        else:
            show_gui_popup(t("syntax_check_title"), t("no_errors_found"))
        return

    error_categories = {
        "Syntax Errors": ["SyntaxError"],
        "File Issues": ["FileNotFound", "EncodingError", "IOError"],
        "Name Issues": ["NameError"],
        "Style Warnings": ["StyleWarning", "IndentationWarning"],
        "Exception Handling": ["ExceptionWarning"],
        "Other Issues": ["AnalysisError"],
    }

    unique_types = sorted(set(e["type"] for e in errors))
    error_text = f"Found {len(errors)} error(s) in {file_path}:\n"
    error_text += f"Error types found: {len(unique_types)}\n\n"


    categorized_errors = {}
    for category, error_types in error_categories.items():
        category_errors = [e for e in errors if e["type"] in error_types]
        if category_errors:
            categorized_errors[category] = category_errors


    for category, category_errors in categorized_errors.items():
        error_text += f"=== {category} ===\n"
        for error in category_errors:
            error_text += f"Line {error['line']}: {error['message']}\n"
            if error.get("context"):
                error_text += f"Context: {error['context']}\n"
            error_text += f"Type: {error['type']}\n\n"

    if IN_JUPYTER:
        display(Markdown(f"**Python Code Analysis**\n\n```\n{error_text}\n```"))
    else:
        show_gui_popup(t("multiple_errors_title"), error_text)


class This:
    """Clase para obtener información del archivo actual"""

    @staticmethod
    def get_caller_path(levels_up: int = 2) -> str:
        """
        Obtiene la ruta del archivo que llamó a esta función
        :param levels_up: Número de niveles en el stack de llamadas a subir (default 2)
        """
        import inspect

        frame = inspect.currentframe()
        try:
            for _ in range(levels_up):
                frame = frame.f_back
                if frame is None:
                    return None

            return frame.f_globals.get("__file__")
        finally:
            del frame

    @staticmethod
    def get_script_path() -> str:
        """Obtiene la ruta del script principal en ejecución"""
        import sys

        return sys.argv[0] if len(sys.argv) > 0 else None


def run(file_path=None):
    def show_error_and_exit():
        show_gui_popup(t("error"), t("provide_file_path"))
        return

    if hasattr(file_path, "__file__"):
        file_path = file_path.__file__

    if file_path is None:
        if len(sys.argv) > 1:
            file_path = sys.argv[1]
        else:
            file_path = This.get_caller_path()

            if file_path is None:
                file_path = This.get_script_path()

    if not file_path or not os.path.exists(file_path):
        show_error_and_exit()
        return

    check_syntax(file_path)



import importlib
import pkgutil

__version__ = "3.0.2"

# Mapeo de nombres ya cargados (para cache)
_loaded_attrs = {}

# --- Descubrir todos los submódulos (core y submodules) ---
# Esto recorre helper/ y helper/submodules/
_package_name = __name__
_package_path = __path__[0]

_all_symbols = set()

def _discover_symbols():
    """
    Descubre todas las funciones, clases y variables públicas de core y submodules
    para ofrecer autocompletado y acceso directo.
    """
    # Primero cargar core
    try:
        core = importlib.import_module(f"{_package_name}.core")
        for name in getattr(core, "__all__", dir(core)):
            if not name.startswith("_"):
                _all_symbols.add(name)
                _loaded_attrs[name] = getattr(core, name)
    except ImportError:
        pass

    # Luego recorrer submodules dinámicamente
    try:
        submodules_pkg = f"{_package_name}.submodules"
        pkg = importlib.import_module(submodules_pkg)
        if hasattr(pkg, "__path__"):
            for _, modname, _ in pkgutil.iter_modules(pkg.__path__, pkg.__name__ + "."):
                try:
                    mod = importlib.import_module(modname)
                    for name in getattr(mod, "__all__", dir(mod)):
                        if not name.startswith("_"):
                            _all_symbols.add(name)
                            _loaded_attrs[name] = getattr(mod, name)
                except ImportError:
                    # Si no se puede importar un submódulo (dependencia opcional faltante), se omite
                    continue
    except ImportError:
        pass

# Descubrimos una vez (no fuerza a importar todo, solo explora)
_discover_symbols()

def __getattr__(name):
    """
    Permite acceder a cualquier símbolo (función, clase, variable) desde helper directamente.
    Lazy load real: si alguien accede a un nombre no cacheado, intentamos cargarlo dinámicamente.
    """
    if name in _loaded_attrs:
        return _loaded_attrs[name]

    # Intentar carga perezosa (por si se agregaron nuevos submódulos)
    for modname in [_package_name + ".core", _package_name + ".submodules"]:
        try:
            mod = importlib.import_module(modname)
            if hasattr(mod, name):
                obj = getattr(mod, name)
                _loaded_attrs[name] = obj
                return obj
        except ImportError:
            continue

    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

def __dir__():
    """
    Ayuda al autocompletado de IDEs.
    """
    return sorted(list(globals().keys()) + list(_all_symbols))



from .submodules.shared import ( ast, asyncio, csv, inspect, json, math, os, re, struct, sys, time, Callable,
    Path, Any, Dict, List, Optional, Set, Tuple, Union, mpl, plt, np, pd,
    sns, sa, FigureCanvasTkAgg, PdfReader, kurtosis, norm, skew, PCA, StandardScaler, Column, MetaData, Table,
    create_engine, func, select, text, SQLAlchemyError, sessionmaker,
    tk, filedialog, messagebox, ttk, ScrolledText, ET,
    gpd, warnings, t, show_gui_popup, show_alert_popup, format_number,
    display, Markdown, IN_JUPYTER, set_language, REGISTRY, register, 
    load_user_translations, GridSpec, datetime)

from .submodules.graph import ( hbar, vbar, pie, boxplot, histo, 
    heatmap, table, scatter, lineplot, kdeplot, 
    jointplot, violinplot, pairplot, countplot, lmplot, 
    swarmplot, regplot, barplot, stripplot)

from .submodules.pyswitch import Switch, switch, AsyncSwitch, async_switch

from .submodules.caller import read, call

from .submodules.statics import get_moda, get_media, get_median, get_rank, get_var, get_desv, disp, IQR

from .submodules.manager import normalize, conditional, convert_file

from .submodules.DBManager import manageDB, DataBase

from .submodules.checker import check_syntax, run, PythonFileChecker

from .submodules.timer import Timer

from .submodules.progress_bar import ProgressBar

from .submodules.resource_monitor import ResourceMonitor

date = datetime.datetime(1997,10,23)
test_df = pd.DataFrame({
    'A': [1, 2, 3, 4, 5],
    'B': [10, 20, 30, 40, 50],
    'C': ['X', 'Y', 'X', 'Z', 'Y'],
    'UUID': '2eb9f3fc-bc5c-41ee-bc27-1f23eff5c279',
    'E': date,
    'id': 636
})


def fig_to_img(fig):
    """Convierte una figura matplotlib a una imagen para mostrar en otro gráfico"""
    fig.canvas.draw()
    img = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
    img = img.reshape(fig.canvas.get_width_height()[::-1] + (3,))
    return img


def generate_all_previews(preview_data):
    preview_title = t("function_preview_title")
    preview_error = t("preview_error_message")

    graph_previews = {}
    for func_name, data in preview_data.items():
        try:
            result = data["preview_func"]()
            if hasattr(result, "figure"):
                graph_previews[func_name] = data
        except:
            pass

    num_funcs = len(graph_previews)
    if num_funcs == 0:
        return None

    rows = (num_funcs + 1) // 2
    fig_height = rows * 6
    fig_width = 18
    fig = plt.figure(figsize=(fig_width, fig_height), tight_layout=True)
    gs = GridSpec(rows, 2, figure=fig)

    for idx, (func_name, data) in enumerate(graph_previews.items()):
        ax = fig.add_subplot(gs[idx // 2, idx % 2])

        try:
            result = data["preview_func"]()
            if hasattr(result, "figure"):
                result.canvas.draw()
                img = np.frombuffer(result.canvas.tostring_rgb(), dtype=np.uint8)
                img = img.reshape(result.canvas.get_width_height()[::-1] + (3,))
                ax.imshow(img)
                ax.axis("off")
                plt.close(result.figure)
        except Exception as e:
            ax.text(
                0.5,
                0.5,
                preview_error.format(error=str(e)),
                ha="center",
                va="center",
                color="red",
            )
            ax.axis("off")

        ax.set_title(f"{func_name}", fontsize=12)

    return fig


def help(type: str = None):

    translations_map = {
        "preview_error": "preview_error",
        "error_in_gui": "error_in_gui",
        "help_error": "help_error",
        "function_preview_title": "function_preview_title",
        "preview_error_message": "preview_error_message",
        "async_preview_not_available": "async_preview_not_available",
        "preview": "preview",
        "example": "example",
        "description": "description",
        "help_available_functions": "help_available_functions",
        "help_usage": "help_usage",
        "title_all": "title_all",
    }

    translated = {}
    for key, value in translations_map.items():
        translated[key] = Switch(value)(
            lambda x: value,
            lambda: t(value),
            "default",
            lambda: value.replace("_", " ").title(),
        )

    help_map = {
        "get_moda": {
            translated["description"]: t("get_moda_description"),
            translated["example"]: "get_moda(np.array([1, 2, 2, 3, 3, 3]), with_repetition=True, decimals=2)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("moda_title"),
                content=str(get_moda(np.array([1, 2, 2, 3, 3, 3]), with_repetition=True, decimals=2)),
                preview_mode=True,
            ),
        },
        "get_media": {
            translated["description"]: t("get_media_description"),
            translated["example"]: "get_media(np.array([1, 2, 3, 4, 5]), nan=False, decimals=2)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("media_title"),
                content=str(get_media(np.array([1, 2, 3, 4, 5]), nan=False, decimals=2)),
                preview_mode=True,
            ),
        },
        "get_median": {
            translated["description"]: t("get_median_description"),
            translated["example"]: "get_median(np.array([1, 2, 3, 4, 5]), nan=False, decimals=2)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("mediana_title"),
                content=str(get_median(np.array([1, 2, 3, 4, 5]), nan=False, decimals=2)),
                preview_mode=True,
            ),
        },
        "get_rank": {
            translated["description"]: t("get_rank_description"),
            translated["example"]: "get_rank(test_df, 'A', decimals=2)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("rango_title"),
                content=str(get_rank(test_df, 'A', decimals=2)),
                preview_mode=True,
            ),
        },
        "get_var": {
            translated["description"]: t("get_var_description"),
            translated["example"]: "get_var(test_df, 'A', decimals=2)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("varianza_title"),
                content=str(get_var(test_df, 'A', decimals=2)),
                preview_mode=True,
            ),
        },
        "get_desv": {
            translated["description"]: t("get_desv_description"),
            translated["example"]: "get_desv(test_df, 'A', decimals=2)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("desviacion_estandar_title"),
                content=str(get_desv(test_df, 'A', decimals=2)),
                preview_mode=True,
            ),
        },
        "disp": {
            translated["description"]: t("disp_description"),
            translated["example"]: "disp(df: pd.DataFrame, column: str, condition: pd.Series = None)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("dispersion_title"),
                content=str(disp(test_df, 'A', condition=test_df['A'] > 2)),
                preview_mode=True,
            ),
        },
        "IQR": {
            translated["description"]: t("IQR_description"),
            translated["example"]: "IQR(df: pd.DataFrame, column: str = None)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("rango_intercuartilico_title"),
                content=str(IQR(test_df, 'A')),
                preview_mode=True,
            ),
        },
        "call": {
            translated["description"]: t("call_description"),
            translated["example"]: "call('filename', type='csv', path='./data', timeout=5)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("call_function_title"),
                content=t("file_read_function_preview"),
                preview_mode=True,
            ),
        },
        "read": {
            translated["description"]: t("read_description"),
            translated["example"]: "read('file.csv')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("read_function_title"),
                content=t("file_read_function_preview"),
                preview_mode=True,
            ),
        },
        "check_syntax": {
            translated["description"]: t("check_syntax_description"),
            translated["example"]: "check_syntax('my_script.py')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("check_syntax_title"),
                content=t("syntax_check_function_preview"),
                preview_mode=True,
            ),
        },
        "run": {
            translated["description"]: t("run_description"),
            translated["example"]: "run('script.py')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("run_function_title"),
                content=t("run_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.exportData": {
            translated["description"]: t("exportData_description"),
            translated["example"]: "db.exportData(table_names='all', format_type='csv')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("export_data_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.addTable": {
            translated["description"]: t("addTable_description"),
            translated["example"]: "db.addTable('new_table', {'col1': 'int', 'col2': 'str'})",
            translated["preview"]: lambda: show_gui_popup(
                title=t("add_table_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.mergeTable": {
            translated["description"]: t("mergeTable_description"),
            translated["example"]: "db.mergeTable('table1', 'table2', on=['id'])",
            translated["preview"]: lambda: show_gui_popup(
                title=t("merge_table_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.join": {
            translated["description"]: t("join_description"),
            translated["example"]: "db.join('inner', 'table1', 'table2', on=['id'])",
            translated["preview"]: lambda: show_gui_popup(
                title=t("join_tables_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.drop": {
            translated["description"]: t("drop_description"),
            translated["example"]: "db.drop('table1', 'table2', cascade=True)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("drop_tables_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.cascadeDelete": {
            translated["description"]: t("cascadeDelete_description"),
            translated["example"]: "db.cascadeDelete('table', 'id = 5')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("cascade_delete_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.recursiveQuery": {
            translated["description"]: t("recursiveQuery_description"),
            translated["example"]: "db.recursiveQuery('employees', 'manager_id IS NULL', 'manager_id = employee_id')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("recursive_query_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.windowFunction": {
            translated["description"]: t("windowFunction_description"),
            translated["example"]: "db.windowFunction('sales', 'ROW_NUMBER', ['region'], ['date'])",
            translated["preview"]: lambda: show_gui_popup(
                title=t("window_function_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.executeRawSQL": {
            translated["description"]: t("executeRawSQL_description"),
            translated["example"]: "db.executeRawSQL('SELECT * FROM table WHERE condition')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("execute_raw_sql_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "DataBase.show": {
            translated["description"]: t("show_description"),
            translated["example"]: "db.show(table_names='all', limit=100)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("show_tables_title"),
                content=t("database_function_preview"),
                preview_mode=True,
            ),
        },
        "hbar": {
            translated["description"]: t("hbar_description"),
            translated["example"]: "hbar( data: pd.Series, title: str, xlabel: str, ylabel: str, save_path: Optional[str] = None, show: bool = True, color: Union[str, List[str]] = 'skyblue', **kwargs)",
            translated["preview"]: lambda: hbar(
                test_df['A'],
                t("ejemplo_hbar_title"),
                t("valores_label"),
                t("categorias_label"),
                show=False,
            ),
        },
        "vbar": {
            translated["description"]: t("vbar_description"),
            translated["example"]: "vbar(test_df['A'], 'Title', 'X Label', 'Y Label', color='skyblue')",
            translated["preview"]: lambda: vbar(
                test_df['A'],
                t("ejemplo_vbar_title"),
                t("categorias_label"),
                t("valores_label"),
                show=False,
            ),
        },
        "pie": {
            translated["description"]: t("pie_description"),
            translated["example"]: "pie([30, 40, 30], ['A', 'B', 'C'], 'Título del Gráfico')",
            translated["preview"]: lambda: pie(
                [30, 40, 30], ["A", "B", "C"], t("ejemplo_pie_title"), show=False
            ),
        },
        "boxplot": {
            translated["description"]: t("boxplot_description"),
            translated["example"]: "boxplot(test_df, x='C', y='A')",
            translated["preview"]: lambda: boxplot(
                test_df, x='C', y='A', title=t("boxplot_example_title"), show=False
            ),
        },
        "histo": {
            translated["description"]: t("histo_description"),
            translated["example"]: "histo(test_df, 'A', bins=5, title='Histograma')",
            translated["preview"]: lambda: histo(
                test_df, 'A', bins=5, title=t("histogram_example_title"), show=False
            ),
        },
        "heatmap": {
            translated["description"]: t("heatmap_description"),
            translated["example"]: "heatmap(test_df.corr(), title='Correlation Heatmap')",
            translated["preview"]: lambda: heatmap(
                test_df[['A', 'B']].corr(), title=t("heatmap_example_title"), show=False
            ),
        },
        "table": {
            translated["description"]: t("table_description"),
            translated["example"]: "table(test_df.head().values, col_labels=test_df.columns)",
            translated["preview"]: lambda: table(
                test_df.head().values,
                col_labels=test_df.columns.tolist(),
                title=t("tabla_ejemplo_title"),
                show=False,
            ),
        },
        "scatter": {
            translated["description"]: t("scatter_description"),
            translated["example"]: "scatter(test_df, x='A', y='B')",
            translated["preview"]: lambda: scatter(
                test_df, x='A', y='B', title=t("scatter_plot_example_title"), show=False
            ),
        },
        "lineplot": {
            translated["description"]: t("lineplot_description"),
            translated["example"]: "lineplot(test_df, x='A', y='B')",
            translated["preview"]: lambda: lineplot(
                test_df, x='A', y='B', title=t("line_plot_example_title"), show=False
            ),
        },
        "kdeplot": {
            translated["description"]: t("kdeplot_description"),
            translated["example"]: "kdeplot(test_df, 'A')",
            translated["preview"]: lambda: kdeplot(
                test_df, 'A', title=t("kde_plot_example_title"), show=False
            ),
        },
        "jointplot": {
            translated["description"]: t("jointplot_description"),
            translated["example"]: "jointplot(test_df, x='A', y='B')",
            translated["preview"]: lambda: jointplot(
                test_df, x='A', y='B', title=t("joint_plot_example_title"), show=False
            ),
        },
        "violinplot": {
            translated["description"]: t("violinplot_description"),
            translated["example"]: "violinplot(test_df, x='C', y='A')",
            translated["preview"]: lambda: violinplot(
                test_df, x='C', y='A', title=t("violin_plot_example_title"), show=False
            ),
        },
        "pairplot": {
            translated["description"]: t("pairplot_description"),
            translated["example"]: "pairplot(test_df, vars=['A', 'B'])",
            translated["preview"]: lambda: pairplot(
                test_df[['A', 'B', 'C']], title=t("pair_plot_example_title"), show=False
            ),
        },
        "countplot": {
            translated["description"]: t("countplot_description"),
            translated["example"]: "countplot(test_df, x='C')",
            translated["preview"]: lambda: countplot(
                test_df, x='C', title=t("count_plot_example_title"), show=False
            ),
        },
        "lmplot": {
            translated["description"]: t("lmplot_description"),
            translated["example"]: "lmplot(test_df, x='A', y='B')",
            translated["preview"]: lambda: lmplot(
                test_df, x='A', y='B', title=t("lm_plot_example_title"), show=False
            ),
        },
        "swarmplot": {
            translated["description"]: t("swarmplot_description"),
            translated["example"]: "swarmplot(test_df, x='C', y='A')",
            translated["preview"]: lambda: swarmplot(
                test_df, x='C', y='A', title=t("swarm_plot_example_title"), show=False
            ),
        },
        "regplot": {
            translated["description"]: t("regplot_description"),
            translated["example"]: "regplot(test_df, x='A', y='B')",
            translated["preview"]: lambda: regplot(
                test_df, x='A', y='B', title=t("reg_plot_example_title"), show=False
            ),
        },
        "barplot": {
            translated["description"]: t("barplot_description"),
            translated["example"]: "barplot(test_df, x='C', y='A')",
            translated["preview"]: lambda: barplot(
                test_df, x='C', y='A', title=t("bar_plot_example_title"), show=False
            ),
        },
        "stripplot": {
            translated["description"]: t("stripplot_description"),
            translated["example"]: "stripplot(test_df, x='C', y='A')",
            translated["preview"]: lambda: stripplot(
                test_df, x='C', y='A', title=t("strip_plot_example_title"), show=False
            ),
        },
        "normalize": {
            translated["description"]: t("normalize_description"),
            translated["example"]: "normalize(np.array([1, 2, 3, 4, 5]))",
            translated["preview"]: lambda: show_gui_popup(
                title=t("normalize_title"),
                content=str(normalize(np.array([1, 2, 3, 4, 5]))),
                preview_mode=True,
            ),
        },
        "conditional": {
            translated["description"]: t("conditional_description"),
            translated["example"]: "conditional(test_df, [test_df['A'] > 2], ['greater'], 'result_col')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("conditional_title"),
                content=str(conditional(test_df, [test_df['A'] > 2], ['greater'], 'result_col').head()),
                preview_mode=True,
            ),
        },
        "convert_file": {
            translated["description"]: t("convert_file_description"),
            translated["example"]: "convert_file('input.shp', 'output.csv')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("convert_file_title"),
                content=t("conversion_function_preview"),
                preview_mode=True,
            ),
        },
        "Switch": {
            translated["description"]: t("Switch_description"),
            translated["example"]: "Switch(value)(case1, action1, case2, action2, 'default', default_action)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("switch_example_title"),
                content=str(Switch(2)(1, t("uno_text"), 2, t("dos_text"), "default", t("otro_text"))),
                preview_mode=True,
            ),
        },
        "switch": {
            translated["description"]: t("switch_description"),
            translated["example"]: "switch(value, {case1, action1, case2, action2, 'default', default_action})",
            translated["preview"]: lambda: show_gui_popup(
                title=t("switch_example_title"),
                content=str(switch(2, {1, t("uno_text"), 2, t("dos_text"), "default", t("otro_text")})),
                preview_mode=True,
            ),
        },
        "AsyncSwitch": {
            translated["description"]: t("AsyncSwitch_description"),
            translated["example"]: "await AsyncSwitch(value)(case1, action1, case2, action2, 'default', default_action)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("async_switch_title"),
                content=t("async_function_preview"),
                preview_mode=True,
            ),
        },
        "async_switch": {
            translated["description"]: t("async_switch_description"),
            translated["example"]: "await async_switch(value, case1, action1, case2, action2, 'default', default_action)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("async_switch_title"),
                content=t("async_function_preview"),
                preview_mode=True,
            ),
        },
        "fig_to_img": {
            translated["description"]: t("fig_to_img_description"),
            translated["example"]: "image_array = fig_to_img(figure)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("fig_to_image_title"),
                content=t("figure_conversion_preview"),
                preview_mode=True,
            ),
        },
        "format_number": {
            translated["description"]: t("format_number_description"),
            translated["example"]: "format_number(1234.5678, decimals=2)",
            translated["preview"]: lambda: show_gui_popup(
                title=t("format_number_title"),
                content=format_number(1234.5678, decimals=2),
                preview_mode=True,
            ),
        },
        "load_user_translations": {
            translated["description"]: t("load_user_translations_description"),
            translated["example"]: "load_user_translations('my_translations.json')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("load_user_translations_title"),
                content=t("translation_load_preview"),
                preview_mode=True,
            ),
        },
        "register": {
            translated["description"]: t("register_description"),
            translated["example"]: "@register('my_function')\ndef my_function(): ...",
            translated["preview"]: lambda: show_gui_popup(
                title=t("register_title"),
                content=t("register_decorator_preview"),
                preview_mode=True,
            ),
        },
        "set_language": {
            translated["description"]: t("set_language_description"),
            translated["example"]: "set_language('es')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("set_language_title"),
                content=t("language_setting_preview"),
                preview_mode=True,
            ),
        },
        "show_gui_popup": {
            translated["description"]: t("show_gui_popup_description"),
            translated["example"]: "show_gui_popup('Título', 'Contenido del mensaje')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("show_gui_popup_example_title"),
                content=t("show_gui_popup_demo"),
                preview_mode=True,
            ),
        },
        "t": {
            translated["description"]: t("t_description"),
            translated["example"]: "translated_text = t('key_name')",
            translated["preview"]: lambda: show_gui_popup(
                title=t("translation_function_title"),
                content=f"{t('example_text')}: {t('help_available_functions')}",
                preview_mode=True,
            ),
        },
    }

    functions = sorted(help_map.keys())

    if type is None:
        if IN_JUPYTER:
            display(Markdown(f"**{t('help_available_functions')}**"))
            for func in functions:
                display(Markdown(f"- `{func}`"))
            display(Markdown(f"\n*{t('help_usage')}*"))
        else:
            func_list = "\n".join(f"- {func}" for func in functions)
            show_gui_popup(
                t("title_all"),
                f"{t('help_available_functions')}\n{func_list}\n\n{t('help_usage')}",
            )
        return

    if not isinstance(type, str):
        msg = t("error_instance_type")
        if IN_JUPYTER:
            display(Markdown(f"**Error:** {msg}"))
        else:
            show_gui_popup(title=t("error_in_gui"), content=msg)
        return

    type = type.lower()

    if type == "all":
        full_doc = []
        preview_data = {}

        for func_name in functions:
            entry = help_map.get(func_name, {})
            doc = entry.get(translated["description"], "")
            example = entry.get(translated["example"], "")

            func_doc = f"{func_name.upper()}\n\n{doc}\n\nExample:\n{example}"
            full_doc.append(func_doc)

            if translated["preview"] in entry:
                preview_data[func_name] = {
                    translated["example"]: example,
                    "preview_func": entry[translated["preview"]],
                }

        full_doc_text = (
            "\n\n"
            + ("=" * 50).join("\n\n")
            + "\n\n".join(full_doc)
            + "\n\n"
            + ("=" * 50)
        )

        if IN_JUPYTER:
            display(Markdown(full_doc_text))
            for func_name, data in preview_data.items():
                display(Markdown(f"**Preview for {func_name}**"))
                try:
                    result = data["preview_func"]()
                    if hasattr(result, "figure"):
                        display(result.figure)
                        plt.close(result.figure)
                    else:
                        display(Markdown(f"```\n{str(result)}\n```"))
                except Exception as e:
                    display(Markdown(f"**Error in preview:**\n```\n{str(e)}\n```"))
        else:
            show_gui_popup(
                t("title_all"),
                full_doc_text,
                plot_function=lambda: generate_all_previews(preview_data),
            )
        return

    if type in functions:
        doc = help_map.get(type, {}).get(translated["description"], t(type))
        entry = help_map.get(type, {})
        example = entry.get(translated["example"], "")
        preview_func = entry.get(translated["preview"])

        if IN_JUPYTER:
            output = f"**{type.upper()}**\n```python\n{doc.strip()}\n```"
            if example:
                output += f"\n\n**{t('example')}:**\n```python\n{example}\n```"
            display(Markdown(output))

            if preview_func:
                try:
                    print(f"\n**{t('preview')}:**")
                    preview_func()
                except Exception as e:
                    display(Markdown(f"**{t('preview_error')}:**\n```\n{str(e)}\n```"))
        else:
            full_text = doc.strip()
            if example:
                full_text += f"\n\n{t('example')}:\n{example}"

            fig = None
            if preview_func:
                try:
                    result = preview_func()
                    if hasattr(result, "figure"):
                        fig = result.figure
                except Exception as e:
                    messagebox.showerror(f"{t('preview_error')} {type}", str(e))

            show_gui_popup(type.upper(), full_text, fig=fig)
    else:
        error_msg = t("help_error").format(type)
        if IN_JUPYTER:
            display(Markdown(f"**{error_msg}**"))
        else:
            show_gui_popup(t("error_in_gui"), error_msg)


__all__ = ["Any", "ast", "async_switch", "asyncio", "AsyncSwitch", "barplot", "boxplot", "call", "Callable", "check_syntax", 
           "Column", "conditional", "convert_file", "countplot", "create_engine", "csv", "DataBase", "datetime", "Dict", "disp", 
           "display", "ET", "fig_to_img", "FigureCanvasTkAgg", "filedialog", "format_number", "func", "generate_all_previews", "get_desv", 
           "get_media", "get_median", "get_moda", "get_rank", "get_var", "gpd", "GridSpec", "hbar", "heatmap", "help", "histo", "IN_JUPYTER", 
           "inspect", "IQR", "jointplot", "json", "kdeplot", "kurtosis", "lineplot", "List", "lmplot", "load_user_translations", "manageDB", 
           "Markdown", "math", "messagebox", "MetaData", "mpl", "norm", "normalize", "np", "Optional", "os", "pairplot", "Path", "PCA", "pd", 
           "PdfReader", "pie", "plt", "PythonFileChecker", "re", "read", "register", "REGISTRY", "regplot", "run", "sa", "scatter", 
           "ScrolledText", "select", "sessionmaker", "Set", "set_language", "show_alert_popup", "show_gui_popup", "skew", "sns", 
           "SQLAlchemyError", "StandardScaler", "stripplot", "struct", "swarmplot", "Switch", "switch", "sys", "t", "Table", "table", "text", 
           "time", "tk", "ttk", "Tuple", "Union", "vbar", "violinplot", "warnings", 
           #v3.0.0 new functions
           "Timer", "ProgressBar", "ResourceMonitor"]



from setuptools import setup, find_packages
import os
import sys
import ast
from pathlib import Path

# -------------------------------
# Configuración general
# -------------------------------
VERSION = "3.0.2"
PACKAGE_NAME = "pyhelper-tools-jbhm"
DESCRIPTION = "A centralized toolkit for Python developers"
AUTHOR = "Juan Braian Hernandez Morani"

# Leer README principal desde la carpeta readme
readme_dir = Path("readme")
main_readme_path = readme_dir / "README.md"
try:
    with open(main_readme_path, "r", encoding="utf-8") as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = "Helper - A centralized toolkit for Python developers"

# -------------------------------
# Definir lenguajes soportados
# -------------------------------
LANGUAGES = [
    "af","sq","am","ar","hy","as","ay","az","bm","eu","be","bn","bho","bs","bg","ca","ceb","ny","zh","co","hr","cs","da","dv",
    "doi","nl","en","eo","et","ee","tl","fi","fr","fy","gl","ka","de","el","gn","gu","ht","ha","haw","iw","hi","hmn","hu","is",
    "ig","ilo","id","ga","it","ja","jw","kn","kk","km","rw","gom","ko","kri","ku","ckb","ky","lo","la","lv","ln","lt","lg","lb",
    "mk","mai","mg","ms","ml","mt","mi","mr","lus","mn","my","ne","no","or","om","ps","fa","pl","pt","pa","qu","ro","ru","sm",
    "sa","gd","nso","sr","st","sn","sd","si","sk","sl","so","es","su","sw","sv","tg","ta","tt","te","th","ti","ts","tr","tk",
    "ak","uk","ur","ug","uz","vi","cy","xh","yi","yo","zu"
]
lang_files = [f"lang/{lang}.json" for lang in LANGUAGES]

# -------------------------------
# Recolector de dependencias
# -------------------------------
try:
    import stdlib_list
    STD_LIB = set(stdlib_list.stdlib_list(sys.version[:3]))
except ImportError:
    # Fallback manual básico
    STD_LIB = {
        "sys","os","re","math","ast","json","time","typing","threading","subprocess",
        "csv","datetime","inspect","struct","pathlib","collections","asyncio","platform",
        "logging","functools","itertools","traceback","queue","concurrent","http",
        "email","unittest","argparse","shutil","signal","ctypes","socket","multiprocessing",
        "uuid","hashlib","copy","contextlib","tempfile","glob","importlib"
    }

NAME_MAP = {
    "sklearn": "scikit-learn",
    "cpuinfo": "py-cpuinfo",
    "IPython": "ipython",
    "sqlalchemy": "SQLAlchemy",
}

INTERNAL_MODULES = {
    "core","submodules","helper","shared","statics","timer","manager","graph","caller","checker",
    "DBManager","pyswitch","progress_bar","complexity_analyzer"
}

def find_imports_in_file(filepath):
    with open(filepath, "r", encoding="utf-8") as f:
        src = f.read()

    try:
        tree = ast.parse(src, filename=filepath)
    except SyntaxError:
        imports = set()
        for line in src.splitlines():
            line = line.lstrip()
            if line.startswith("import ") or line.startswith("from "):
                parts = line.split()
                if parts[0] == "import":
                    imports.add(parts[1].split(".")[0])
                elif parts[0] == "from":
                    imports.add(parts[1].split(".")[0])
        return imports, []

    imports = set()
    conditional = []

    class Visitor(ast.NodeVisitor):
        def visit_FunctionDef(self, node): return
        def visit_ClassDef(self, node): return

        def visit_If(self, node):
            test_src = ast.unparse(node.test) if hasattr(ast, "unparse") else ""
            platforms = []
            if any(x in test_src for x in ["platform.system", "sys.platform", "os.name"]):
                for n in ast.walk(node.test):
                    if isinstance(n, ast.Constant) and isinstance(n.value, str):
                        platforms.append(n.value)
            for subnode in node.body:
                if isinstance(subnode, ast.Import):
                    for alias in subnode.names:
                        pkg = alias.name.split(".")[0]
                        if platforms:
                            conditional.append((pkg, platforms))
                        else:
                            imports.add(pkg)
                elif isinstance(subnode, ast.ImportFrom) and subnode.module:
                    pkg = subnode.module.split(".")[0]
                    if platforms:
                        conditional.append((pkg, platforms))
                    else:
                        imports.add(pkg)
            self.generic_visit(node)

        def visit_Import(self, node):
            for alias in node.names:
                imports.add(alias.name.split(".")[0])

        def visit_ImportFrom(self, node):
            if node.module:
                imports.add(node.module.split(".")[0])

    Visitor().visit(tree)
    return imports, conditional

def collect_all_imports(target_dir="helper"):
    raw_imports = set()
    conditional_map = {}
    for root, _, files in os.walk(target_dir):
        for file in files:
            if file.endswith(".py"):
                path = os.path.join(root, file)
                imports, conditionals = find_imports_in_file(path)
                raw_imports.update(imports)
                for pkg, platforms in conditionals:
                    conditional_map.setdefault(pkg, set()).update(platforms)
    return sorted(raw_imports), conditional_map

raw_deps, conditional_map = collect_all_imports("helper")

base_requires = []
for pkg in raw_deps:
    if pkg in STD_LIB or pkg in INTERNAL_MODULES:
        continue
    base_requires.append(NAME_MAP.get(pkg, pkg))

base_requires.append("stdlib-list")

platform_marker_deps = []
for pkg, platforms in conditional_map.items():
    if pkg in STD_LIB or pkg in INTERNAL_MODULES:
        continue
    pip_name = NAME_MAP.get(pkg, pkg)
    platforms = [p.lower() for p in platforms]
    if "linux" in platforms:
        platform_marker_deps.append(f"{pip_name}; sys_platform == 'linux'")
    if "windows" in platforms:
        platform_marker_deps.append(f"{pip_name}; sys_platform == 'win32'")
    if "darwin" in platforms:
        platform_marker_deps.append(f"{pip_name}; sys_platform == 'darwin'")

if "psycopg2" in base_requires:
    base_requires = ["psycopg2-binary" if p == "psycopg2" else p for p in base_requires]

install_requires = sorted(set(base_requires + platform_marker_deps))
install_requires.remove("pkgutil")

extras_require = {
    "linux": ["pyamdgpuinfo"],
    "windows": ["wmi"],
    "macos": []
}

# -------------------------------
# Pre-flight check (warnings)
# -------------------------------
all_pip_names = set(NAME_MAP.values()) | set(install_requires)
unknowns = []
for dep in base_requires:
    if dep not in all_pip_names:
        unknowns.append(dep)

if unknowns:
    print("\n[WARNING] The following dependencies were detected but not confirmed in NAME_MAP or install_requires:")
    for dep in unknowns:
        print(f"   - {dep}")
    print("Make sure these are valid PyPI package names before uploading.\n")

# -------------------------------
# Archivos adicionales
# -------------------------------
readme_files = []
if readme_dir.exists():
    for file in readme_dir.iterdir():
        if file.is_file() and file.suffix in [".md", ".txt", ".rst"]:
            readme_files.append(f"readme/{file.name}")

# -------------------------------
# Setup final
# -------------------------------
setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    package_data={
        "helper": lang_files + readme_files
    },
    include_package_data=True,
    install_requires=install_requires,
    extras_require=extras_require,
    python_requires=">=3.8"
)



from typing import Callable, Any, Dict, List
import pandas as pd
import numpy as np
import re
import inspect

class BaseSwitch:
    """Clase base con la lógica común para ambos tipos de switch"""
    
    def __init__(self, value: Any, *, match_all: bool = False, debug: bool = False):
        self.value = value
        self.match_all = match_all
        self.debug = debug

    def _match_case(self, key: Any) -> bool:
        """Lógica común para comparación de casos"""
        try:
            if self.value == key:
                return True

            if isinstance(key, re.Pattern):
                return bool(key.match(str(self.value)))

            if isinstance(key, type):
                return isinstance(self.value, key)

            if isinstance(key, Callable):
                return key(self.value)

            if isinstance(self.value, (pd.Series, pd.DataFrame, np.ndarray)):
                return str(self.value) == str(key)

        except Exception as e:
            if self.debug:
                print(f"[Switch] Error matching case {key}: {e}")
            return False

        return False

    def _convert_dict_to_pairs(self, case_dict: Dict) -> List:
        """Convierte formato de diccionario a pares de casos"""
        pairs = []
        try:

            if 'cases' in case_dict:
                for case in case_dict.get('cases', []):
                    pairs.extend([case['case'], case['then']])
                if 'default' in case_dict:
                    pairs.extend(['default', case_dict['default']])
            

            else:
                for case_key, action in case_dict.items():
                    if case_key == "default":
                        pairs.extend(['default', action])
                    else:
                        pairs.extend([case_key, action])
                        
        except (KeyError, TypeError, AttributeError) as e:
            raise ValueError(f"Invalid dictionary format: {e}") from e
            
        return pairs

    def _parse_cases(self, *cases: Any) -> List:
        """Parsea casos en diferentes formatos"""
        if not cases:
            raise ValueError("At least one case must be provided")
            

        if len(cases) == 1 and isinstance(cases[0], dict):
            return self._convert_dict_to_pairs(cases[0])
        

        if len(cases) == 1 and isinstance(cases[0], (list, tuple)):
            return list(cases[0])
        

        return list(cases)

    def _validate_cases(self, cases: List) -> None:
        """Valida que los casos tengan el formato correcto"""
        if len(cases) % 2 != 0:
            raise ValueError("Cases must be defined in pairs: (condition, action)")
        

        default_count = cases.count('default')
        if default_count > 1:
            raise ValueError("Only one default case is allowed")
        

        if 'default' in cases:
            default_index = cases.index('default')
            if default_index % 2 != 0:
                raise ValueError("Default case must be a condition (even index)")

    def _log_match(self, condition: Any, switch_type: str) -> None:
        """Registra coincidencias en modo debug"""
        if self.debug:
            print(f"[{switch_type}] Matched case: {repr(condition)}")
            print(f"[{switch_type}] Value: {repr(self.value)}")

    def _log_default(self, switch_type: str) -> None:
        """Registra ejecución de default en modo debug"""
        if self.debug:
            print(f"[{switch_type}] Executing default case")


class Switch(BaseSwitch):
    """Implementación sincrónica del switch con múltiples formatos"""
    
    def __call__(self, *cases: Any) -> Any:
        """
        Ejecuta el switch con diferentes formatos:
        
        Formato 1: Pares tradicionales
            Switch(value)(cond1, action1, cond2, action2, default, default_action)
        
        Formato 2: Diccionario estructurado
            Switch(value)({
                "cases": [
                    {"case": cond1, "then": action1},
                    {"case": cond2, "then": action2}
                ],
                "default": default_action
            })
        
        Formato 3: Diccionario directo
            Switch(value)({cond1: action1, cond2: action2, "default": default_action})
        """
        try:
            parsed_cases = self._parse_cases(*cases)
            self._validate_cases(parsed_cases)
            
            matched_any = False
            results = []

            for i in range(0, len(parsed_cases), 2):
                condition = parsed_cases[i]
                action = parsed_cases[i + 1]

                if condition == "default":
                    continue

                if self._match_case(condition):
                    matched_any = True
                    self._log_match(condition, "Switch")

                    result = self._run_action(action)
                    results.append(result)
                    
                    if not self.match_all:
                        return result


            if not matched_any or self.match_all:
                default_result = self._handle_default(parsed_cases, matched_any)
                if default_result is not None:
                    if self.match_all:
                        results.append(default_result)
                        return results
                    return default_result

            if self.match_all:
                return results
                
            raise ValueError(f"No matching case found for: {repr(self.value)}")

        except Exception as e:
            if self.debug:
                print(f"[Switch] Error: {e}")
            raise

    def _handle_default(self, cases: List, matched_any: bool) -> Any:
        """Maneja la ejecución del caso default"""
        if "default" in cases:
            idx = cases.index("default")
            if not matched_any or self.match_all:
                self._log_default("Switch")
                return self._run_action(cases[idx + 1])
        return None

    def _run_action(self, action: Any) -> Any:
        """Ejecuta una acción sincrónica"""
        try:
            if callable(action):
                return action()
            return action
        except Exception as e:
            if self.debug:
                print(f"[Switch] Error executing action: {e}")
            raise


class AsyncSwitch(BaseSwitch):
    """Implementación asincrónica del switch con múltiples formatos"""
    
    async def __call__(self, *cases: Any) -> Any:
        """Versión asincrónica del switch con los mismos formatos"""
        try:
            parsed_cases = self._parse_cases(*cases)
            self._validate_cases(parsed_cases)
            
            matched_any = False
            results = []

            for i in range(0, len(parsed_cases), 2):
                condition = parsed_cases[i]
                action = parsed_cases[i + 1]

                if condition == "default":
                    continue

                if self._match_case(condition):
                    matched_any = True
                    self._log_match(condition, "AsyncSwitch")

                    result = await self._run_action(action)
                    results.append(result)
                    
                    if not self.match_all:
                        return result


            if not matched_any or self.match_all:
                default_result = await self._handle_default(parsed_cases, matched_any)
                if default_result is not None:
                    if self.match_all:
                        results.append(default_result)
                        return results
                    return default_result

            if self.match_all:
                return results
                
            raise ValueError(f"No matching case found for: {repr(self.value)}")

        except Exception as e:
            if self.debug:
                print(f"[AsyncSwitch] Error: {e}")
            raise

    async def _handle_default(self, cases: List, matched_any: bool) -> Any:
        """Maneja la ejecución del caso default asincrónico"""
        if "default" in cases:
            idx = cases.index("default")
            if not matched_any or self.match_all:
                self._log_default("AsyncSwitch")
                return await self._run_action(cases[idx + 1])
        return None

    async def _run_action(self, action: Any) -> Any:
        """Ejecuta una acción asincrónica"""
        try:
            if callable(action):
                if inspect.iscoroutinefunction(action):
                    return await action()
                result = action()
                return await result if inspect.isawaitable(result) else result
            return action
        except Exception as e:
            if self.debug:
                print(f"[AsyncSwitch] Error executing action: {e}")
            raise



def switch(value: Any, *cases: Any, **kwargs: Any) -> Any:
    """Función wrapper para uso más natural del switch"""
    return Switch(value, **kwargs)(*cases)

async def async_switch(value: Any, *cases: Any, **kwargs: Any) -> Any:
    """Función wrapper asincrónica para uso más natural"""
    return await AsyncSwitch(value, **kwargs)(*cases)



from .shared import time, tk, timedelta, Optional

class Timer:
    def __init__(self, parent_window=None):
        self.start_time: Optional[float] = None
        self.elapsed_time: float = 0
        self.paused: bool = False
        self.pause_start: Optional[float] = None
        

        self.window = tk.Toplevel() if parent_window is None else parent_window
        self.window.title("Timer")
        self.window.geometry("350x200")
        self.window.resizable(False, False)
        self.window.configure(bg='#f0f0f0')
        

        self.dark_mode = False
        self.colors = {
            'bg': '#f0f0f0',
            'fg': '#000000',
            'accent': '#007acc',
            'secondary': '#e0e0e0'
        }
        
        self._create_widgets()
        self._update_display()
    
    def _create_widgets(self):

        main_frame = tk.Frame(self.window, bg=self.colors['bg'], padx=20, pady=20)
        main_frame.pack(fill=tk.BOTH, expand=True)
        

        self.time_var = tk.StringVar(value="00:00:00")
        time_font = ("Arial", 32, "bold")
        self.time_label = tk.Label(main_frame, textvariable=self.time_var, 
                                  font=time_font, bg=self.colors['bg'], fg=self.colors['accent'])
        self.time_label.pack(pady=10)
        

        btn_frame = tk.Frame(main_frame, bg=self.colors['bg'])
        btn_frame.pack(pady=10)
        

        self.start_btn = tk.Button(btn_frame, text="Start", command=self.start,
                                  bg=self.colors['secondary'], fg=self.colors['fg'],
                                  relief=tk.FLAT, padx=15, pady=5)
        self.start_btn.pack(side=tk.LEFT, padx=5)
        
        self.pause_btn = tk.Button(btn_frame, text="Pause", command=self.pause, state=tk.DISABLED,
                                  bg=self.colors['secondary'], fg=self.colors['fg'],
                                  relief=tk.FLAT, padx=15, pady=5)
        self.pause_btn.pack(side=tk.LEFT, padx=5)
        
        self.reset_btn = tk.Button(btn_frame, text="Reset", command=self.reset,
                                  bg=self.colors['secondary'], fg=self.colors['fg'],
                                  relief=tk.FLAT, padx=15, pady=5)
        self.reset_btn.pack(side=tk.LEFT, padx=5)
        

        self.theme_btn = tk.Button(main_frame, text="🌙", command=self.toggle_theme,
                                  bg=self.colors['secondary'], fg=self.colors['fg'],
                                  relief=tk.FLAT, font=("Arial", 12))
        self.theme_btn.pack(side=tk.BOTTOM, pady=5)
    
    def toggle_theme(self):
        self.dark_mode = not self.dark_mode
        if self.dark_mode:
            self.colors = {
                'bg': '#2d2d2d',
                'fg': '#ffffff',
                'accent': '#4ec9b0',
                'secondary': '#3d3d3d'
            }
            self.theme_btn.config(text="☀️")
        else:
            self.colors = {
                'bg': '#f0f0f0',
                'fg': '#000000',
                'accent': '#007acc',
                'secondary': '#e0e0e0'
            }
            self.theme_btn.config(text="🌙")
        

        self.window.configure(bg=self.colors['bg'])
        for widget in self.window.winfo_children():
            if isinstance(widget, tk.Frame):
                widget.configure(bg=self.colors['bg'])
        
        self.time_label.configure(bg=self.colors['bg'], fg=self.colors['accent'])
        
        for btn in [self.start_btn, self.pause_btn, self.reset_btn, self.theme_btn]:
            btn.configure(bg=self.colors['secondary'], fg=self.colors['fg'])
    
    def start(self):
        if self.paused and self.pause_start is not None:
            pause_duration = time.time() - self.pause_start
            self.start_time += pause_duration
            self.paused = False
            self.pause_start = None
        elif self.start_time is None:
            self.start_time = time.time()
            self.paused = False
        
        self.start_btn.config(state=tk.DISABLED)
        self.pause_btn.config(state=tk.NORMAL)
    
    def pause(self):
        if not self.paused and self.start_time is not None:
            self.paused = True
            self.pause_start = time.time()
            self.elapsed_time = self.pause_start - self.start_time
        
        self.start_btn.config(state=tk.NORMAL)
        self.pause_btn.config(state=tk.DISABLED)
    
    def reset(self):
        self.start_time = None
        self.elapsed_time = 0
        self.paused = False
        self.pause_start = None
        
        self.start_btn.config(state=tk.NORMAL)
        self.pause_btn.config(state=tk.DISABLED)
        self._update_display()
    
    def get_elapsed_time(self) -> float:
        if self.paused:
            return self.elapsed_time
        elif self.start_time is not None:
            return time.time() - self.start_time
        else:
            return 0
    
    def get_formatted_time(self) -> str:
        elapsed = self.get_elapsed_time()
        return str(timedelta(seconds=int(elapsed)))
    
    def is_running(self) -> bool:
        return self.start_time is not None and not self.paused
    
    def _update_display(self):
        elapsed = self.get_elapsed_time()
        formatted = str(timedelta(seconds=int(elapsed)))
        self.time_var.set(formatted)
        
        if self.is_running():
            self.window.after(1000, self._update_display)
